/*--------------------------------------------------------------------------
 * Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 * OTHER DEALINGS IN THE SOFTWARE.
 * 
 * Except as contained in this notice, the name of Dallas Semiconductor 
 * shall not be used except as stated in the Dallas Semiconductor 
 * Branding Policy. 
 *--------------------------------------------------------------------------
 *
 * Version 3.20A
 * History: 3.11 -> 3.20A Corrected prototype for TMGetAdapterSpec.
 */

/* includes */
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <windows.h>
#ifndef _WIN32_WCE
   #include <stdio.h>
   #include <conio.h>
   #include <dos.h>
   #include <fcntl.h>
   #include <io.h>
   #include <time.h>
#endif

/* type defs */
typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned long  ulong;

/* typedef structure sent back by TMEX routines */
typedef struct      
{
   uchar name[4];
   uchar extension;
   uchar startpage;
   uchar numpages;
   uchar attrib;
   uchar bitmap[32];
} FileEntry;

/* structure to hold directory path */
typedef struct
{
   uchar NumEntries;      /* number of entries in path 0-10 */
   char  Ref;             /* reference character '\' or '.' */
   char  Entries[10][4];  /* sub-directory entry names */                                          
} DirectoryPath;

/* Holds info of each object in the Directory list */
struct DirNumInfo 
{  
   unsigned char Name[4];
   unsigned char Extension;
   char Attrib;
};

/* structure to hold exportable device parameters by family code (3.11) */
typedef struct
{
   short features[32];
   char  dscrptn[255];
} Specification;


/* Error codes from TMEX functions */
#define    NO_DEVICE             -1 
#define    WRONG_TYPE            -2 
#define    FILE_READ_ERR         -3 
#define    BUFFER_TOO_SMALL      -4 
#define    HANDLE_NOT_AVAIL      -5  
#define    FILE_NOT_FOUND        -6 
#define    REPEAT_FILE           -7 
#define    HANDLE_NOT_USED       -8 
#define    FILE_WRITE_ONLY       -9 
#define    OUT_OF_SPACE          -10 
#define    FILE_WRITE_ERR        -11 
#define    FILE_READ_ONLY        -12 
#define    FUNC_NOT_SUP          -13 
#define    BAD_FILENAME          -14 
#define    CANT_DEL_READ_ONLY    -15 
#define    HANDLE_NOT_EXIST      -16 
#define    ONE_WIRE_PORT_ERROR   -17 
#define    INVALID_DIRECTORY     -18 
#define    DIRECTORY_NOT_EMPTY   -19 
#define    UNABLE_TO_CREATE_DIR  -20 
#define    NO_PROGRAM_JOB        -21 
#define    PROGRAM_WRITE_PROTECT -22
#define    NON_PROGRAM_PARTS     -23 
#define    ADDFILE_TERMINATED    -24 
#define    TIMEOUT               -25 
#define    INVALID_ARGUMENT      -26 
#define    BAD_ACK               -27 
#define    INVALID_SESSION       -200
#define    NO_BASDRV_FOUND       -201

/* Basic physical level error codes  */
#define    BPORT_NOT_INITIALIZED   -1 
#define    BPORT_NOT_EXIST         -2 
#define    BNO_SUCH_FUNCTION       -3 

/* Transport level error codes  */
#define    TERROR_READ_WRITE       -4 
#define    TBUFFER_TOO_SMALL       -5 
#define    TDEVICE_TOO_SMALL       -6 
#define    TNO_DEVICE              -7 
#define    TBLOCK_TOO_BIG          -8 
#define    TWRONG_TYPE             -9 
#define    TPAGE_REDIRECTED        -10
#define    TPROGRAM_NOT_POSSIBLE   -11
#define    BCOM_FAILURE            -12
#define    BCOM_EVENT              -13

/* for TMOneWireLevel */
#define    LEVEL_NORMAL            0
#define    LEVEL_STRONG_PULLUP     1
#define    LEVEL_BREAK             2
#define    LEVEL_PROGRAM           3  
#define    PRIMED_NONE             0
#define    PRIMED_BIT              1
#define    PRIMED_BYTE             2        
#define    LEVEL_READ              1
#define    LEVEL_SET               0

/* misc */
#define    GENERAL_FAIL            -2 
#define    BAD_ARGUMENT            50
#define    NO_DRIVERS              51
#define    KEY_ABORT               52
#define    OUT_MEMORY              53
#define    NORMAL_EXIT             0

#define    TRUE                    1
#define    FALSE                   0
#define    DIR_READ                1
#define    DIR_SET                 0
#define    DIR_REMOVE              1
#define    DIR_MAKE                0

/* for specification */
#define    NOMEM                   0
#define    NVRAM                   1
#define    EPROM1                  2
#define    EPROM2                  3
#define    EPROM3                  4
#define    EEPROM1                 5
#define    MNVRAM                  6
#define    EEPROM2                 7
#define    NVRAM2                  8
#define    NVRAM3                  9 
#define    FTR_REG_PAGES           0
#define    FTR_REG_LEN             1
#define    FTR_STAT_PAGES          2
#define    FTR_STAT_LEN            3
#define    FTR_MAX_COM             4 
#define    FTR_MEM_TYPE            5

#ifdef __cplusplus
   extern "C" {
// TMEX - Sesssion
 __declspec(dllimport) long pascal  TMExtendedStartSession(short, short, void *);
 __declspec(dllimport) long pascal  TMStartSession(short);
 __declspec(dllimport) short pascal TMValidSession(long);
 __declspec(dllimport) short pascal TMEndSession(long);
// TMEX - File Operations
 __declspec(dllimport) short pascal TMFirstFile(long, void *, FileEntry *);
 __declspec(dllimport) short pascal TMNextFile(long, void *, FileEntry *);
 __declspec(dllimport) short pascal TMOpenFile(long, void *, FileEntry *);
 __declspec(dllimport) short pascal TMCreateFile(long, void *, short *, FileEntry *);
 __declspec(dllimport) short pascal TMCloseFile(long, void *, short);
 __declspec(dllimport) short pascal TMReadFile(long, void *, short, unsigned char*, short);
 __declspec(dllimport) short pascal TMWriteFile(long, void *, short, unsigned char *, short);
 __declspec(dllimport) short pascal TMDeleteFile(long, void *, FileEntry *);
 __declspec(dllimport) short pascal TMFormat(long, void *);
 __declspec(dllimport) short pascal TMAttribute(long, void *, short, FileEntry *);
 __declspec(dllimport) short pascal TMReNameFile(long, void *, short, FileEntry *);
 __declspec(dllimport) short pascal TMChangeDirectory(long, void *, short, DirectoryPath *);
 __declspec(dllimport) short pascal TMDirectoryMR(long, void *, short, FileEntry *);
 __declspec(dllimport) short pascal TMCreateProgramJob(long, void *);
 __declspec(dllimport) short pascal TMDoProgramJob(long, void *);
 __declspec(dllimport) short pascal TMWriteAddFile(long, void *, short, short, short, unsigned char *, short);
 __declspec(dllimport) short pascal TMTerminateAddFile(long, void *, FileEntry *);
 __declspec(dllimport) short pascal TMGetFamilySpec(long, void *, Specification *);  /* (3.11) */
// TMEX - Transport
 __declspec(dllimport) short pascal TMReadPacket(long, void *, short, unsigned char *, short);
 __declspec(dllimport) short pascal TMWritePacket(long, void *, short, unsigned char *, short);
 __declspec(dllimport) short pascal TMBlockIO(long, void *, unsigned char *, short);
 __declspec(dllimport) short pascal TMExtendedReadPage(long, void *, short, unsigned char *, short);
 __declspec(dllimport) short pascal TMProgramByte(long, void *, short, short, short, short *, short);
 __declspec(dllimport) short pascal TMProgramBlock(long, void far *, uchar far *, short, short, short far *); /* (3.11) */
 __declspec(dllimport) long  pascal TMCRC(short, uchar far *, ushort, short);  /* (3.11) */
// TMEX - Network
 __declspec(dllimport) short pascal TMFirst(long, void *);
 __declspec(dllimport) short pascal TMNext(long, void *);
 __declspec(dllimport) short pascal TMAccess(long, void *);
 __declspec(dllimport) short pascal TMStrongAccess(long, void *);
 __declspec(dllimport) short pascal TMStrongAlarmAccess(long, void *);
 __declspec(dllimport) short pascal TMOverAccess(long, void *);
 __declspec(dllimport) short pascal TMRom(long, void *, short *);
 __declspec(dllimport) short pascal TMFirstAlarm(long, void *);
 __declspec(dllimport) short pascal TMNextAlarm(long, void *);
 __declspec(dllimport) short pascal TMFamilySearchSetup(long, void *, short);
 __declspec(dllimport) short pascal TMSkipFamily(long, void *);
 __declspec(dllimport) short pascal TMAutoOverDrive(long, void *, short);
 __declspec(dllimport) short pascal TMSearch(short, short, short, short); /* (3.11) */
// TMEX - Hardware Specific
 __declspec(dllimport) short pascal TMSetup(long);
 __declspec(dllimport) short pascal TMTouchReset(long);
 __declspec(dllimport) short pascal TMTouchByte(long, short);
 __declspec(dllimport) short pascal TMTouchBit(long, short);
 __declspec(dllimport) short pascal TMProgramPulse(long);
 __declspec(dllimport) short pascal TMClose(long);
 __declspec(dllimport) short pascal TMOneWireCom(long, short, short);
 __declspec(dllimport) short pascal TMOneWireLevel(long, short, short, short);
 __declspec(dllimport) short pascal TMGetTypeVersion(short, char *);
 __declspec(dllimport) short pascal Get_Version(char *);
 __declspec(dllimport) short pascal TMBlockStream(long, void *, unsigned char *, short);
 __declspec(dllimport) short pascal TMGetAdapterSpec(long, Specification far *);  /* (3.11) */
 __declspec(dllimport) short pascal TMReadDefaultPort(short far *, short far *);  /* (3.11) */
//---------------------------------------------------------------------------
  }
#else
// session
extern long  far pascal TMExtendedStartSession(short, short, void far *);
extern short far pascal TMValidSession(long); 
extern short far pascal TMEndSession(long); 
extern short far pascal Get_Version(char far *); 
// file_operations
extern short far pascal TMFirstFile(long, void far *, FileEntry far *); 
extern short far pascal TMNextFile(long, void far *, FileEntry far *);
extern short far pascal TMOpenFile(long, void far *, FileEntry far *);
extern short far pascal TMCreateFile(long, void far *, short far *, FileEntry far *);
extern short far pascal TMCloseFile(long, void far *, short);
extern short far pascal TMReadFile(long, void far *, short, uchar far *, short);
extern short far pascal TMWriteFile(long, void far *, short, uchar far *, short);
extern short far pascal TMDeleteFile(long, void far *, FileEntry far *);
extern short far pascal TMFormat(long, void far *);
extern short far pascal TMAttribute(long, void far *, short, FileEntry far *);
extern short far pascal TMReNameFile(long, void far *, short, FileEntry far *); 
extern short far pascal TMChangeDirectory(long, void far *, short, DirectoryPath far *); 
extern short far pascal TMDirectoryMR(long, void far *, short, FileEntry far *);
extern short far pascal TMCreateProgramJob(long, void far *);
extern short far pascal TMDoProgramJob(long, void far *);
extern short far pascal TMWriteAddFile(long, void far *, short, short, short, uchar far *, short);
extern short far pascal TMTerminateAddFile(long, void far *, FileEntry far *);
extern short far pascal TMGetFamilySpec(long, void far *, Specification *); /* (3.11) */
// transport
extern short far pascal TMReadPacket(long, void far *, short, uchar far *, short);
extern short far pascal TMWritePacket(long, void far *, short, uchar far *, short);
extern short far pascal TMBlockIO(long, uchar far *, short);
extern short far pascal TMExtendedReadPage(long, void far *, short, uchar far *, short);
extern short far pascal TMProgramByte(long, void far *, short, short, short, short far *, short);
extern short far pascal TMProgramBlock(long, void far *, uchar far *, short, short, short far *); /* (3.11) */
extern long  far pascal TMCRC(short, uchar far *, ushort, short);  /* (3.11) */
// network
extern short far pascal TMSkipFamily(long, void far *); 
extern short far pascal TMFamilySearchSetup(long, void far *, short); 
extern short far pascal TMFirst(long, void far *);
extern short far pascal TMNext(long, void far *);
extern short far pascal TMAccess(long, void far *);
extern short far pascal TMOverAccess(long, void far *);
extern short far pascal TMStrongAccess(long, void far *);
extern short far pascal TMStrongAlarmAccess(long, void far *);
extern short far pascal TMRom(long, void far *, short far *);
extern short far pascal TMFirstAlarm(long, void far *);
extern short far pascal TMNextAlarm(long, void far *);  
extern short far pascal TMAutoOverDrive(long, void far *, short);  
extern short far pascal TMSearch(short, short, short, short); /* (3.11) */
// hardware_specific
extern short far pascal TMSetup(long);
extern short far pascal TMTouchByte(long, short);
extern short far pascal TMTouchReset(long);
extern short far pascal TMTouchBit(long, short);
extern short far pascal TMClose(long);
extern short far pascal TMProgramPulse(long); 
extern short far pascal TMOneWireCom(long, short, short); 
extern short far pascal TMOneWireLevel(long, short, short, short); 
extern short far pascal TMGetTypeVersion(short,char far *); 
extern short far pascal TMBlockStream(long, uchar far *, short);
extern short far pascal TMGetAdapterSpec(long, Specification far *);  /* 3.11 */
extern short far pascal TMReadDefaultPort(short far *, short far *);  /* 3.11 */

#endif


